"""Step 3: Abbreviate & Symbolize - Comprehensive technical abbreviations, symbols, and acronyms."""

import re


# Technical abbreviations (expanded common software/business terms)
_TECHNICAL_ABBREVIATIONS = {
    "information": "info",
    "documentation": "docs", 
    "document": "doc",
    "configuration": "config",
    "environment": "env",
    "development": "dev",
    "production": "prod",
    "application": "app",
    "database": "DB",
    "repository": "repo",
    "directory": "dir",
    "function": "func",
    "variable": "var",
    "parameter": "param",
    "argument": "arg",
    "maximum": "max",
    "minimum": "min",
    "average": "avg",
    "percent": "%",
    "percentage": "%",
    "number": "num",
    "message": "msg",
    "error": "err",
    "warning": "warn",
    "example": "eg",
    "versus": "vs",
    "et cetera": "etc",
    "and so on": "etc",
    "attribute": "attr",
    "property": "prop",
    "method": "fn",
    "object": "obj",
    "string": "str",
    "integer": "int",
    "boolean": "bool",
    "length": "len",
    "count": "cnt",
    "index": "idx",
    "identifier": "id",
    "reference": "ref",
    "requirements": "reqs",
    "specification": "spec",
    "implementation": "impl",
    "alternative": "alt",
    "administrator": "admin",
    "administration": "admin",
    "automatic": "auto",
    "automatically": "auto",
    "background": "bg",
    "business": "biz",
    "commercial": "comm",
    "communication": "comm",
    "community": "comm",
    "company": "co",
    "corporation": "corp",
    "coordinate": "coord",
    "coordinates": "coords",
    "current": "curr",
    "customer": "cust",
    "default": "def",
    "definition": "def",
    "department": "dept",
    "description": "desc",
    "difference": "diff",
    "different": "diff",
    "duplicate": "dup",
    "electric": "elec",
    "electronic": "elec",
    "equivalent": "equiv",
    "especially": "esp",
    "estimate": "est",
    "evaluation": "eval",
    "executive": "exec",
    "extension": "ext",
    "external": "ext",
    "favorite": "fav",
    "frequency": "freq",
    "general": "gen",
    "generate": "gen",
    "government": "gov",
    "hardware": "hw",
    "software": "sw",
    "history": "hist",
    "horizontal": "horiz",
    "vertical": "vert",
    "image": "img",
    "including": "incl",
    "increase": "inc",
    "individual": "indiv",
    "installation": "install",
    "instruction": "instr",
    "interface": "iface",
    "internal": "int",
    "international": "intl",
    "introduction": "intro",
    "language": "lang",
    "location": "loc",
    "machine": "mach",
    "management": "mgmt",
    "manager": "mgr",
    "manufacturer": "mfg",
    "marketing": "mktg",
    "mathematics": "math",
    "memory": "mem",
    "modification": "mod",
    "monitor": "mon",
    "multiple": "multi",
    "network": "net",
    "operation": "op",
    "operations": "ops",
    "optional": "opt",
    "organization": "org",
    "original": "orig",
    "package": "pkg",
    "page": "pg",
    "password": "pwd",
    "performance": "perf",
    "personal": "pers",
    "platform": "plat",
    "position": "pos",
    "possible": "poss",
    "previous": "prev",
    "primary": "pri",
    "priority": "pri",
    "private": "priv",
    "problem": "prob",
    "procedure": "proc",
    "process": "proc",
    "professional": "prof",
    "program": "prog",
    "project": "proj",
    "protocol": "proto",
    "public": "pub",
    "quality": "qual",
    "quantity": "qty",
    "question": "q",
    "questions": "qs",
    "receive": "recv",
    "recent": "rec",
    "recommend": "rec",
    "record": "rec",
    "register": "reg",
    "regular": "reg",
    "relationship": "rel",
    "relative": "rel",
    "remove": "rm",
    "replace": "repl",
    "report": "rpt",
    "request": "req",
    "required": "req",
    "resource": "res",
    "response": "resp",
    "responsible": "resp",
    "result": "res",
    "review": "rev",
    "revision": "rev",
    "schedule": "sched",
    "science": "sci",
    "scientific": "sci",
    "search": "srch",
    "second": "sec",
    "secondary": "sec",
    "section": "sect",
    "security": "sec",
    "select": "sel",
    "sequence": "seq",
    "series": "ser",
    "service": "svc",
    "session": "sess",
    "similar": "sim",
    "simulate": "sim",
    "solution": "sol",
    "source": "src",
    "special": "spec",
    "specific": "spec",
    "standard": "std",
    "statistics": "stats",
    "status": "stat",
    "structure": "struct",
    "student": "stud",
    "subject": "subj",
    "summary": "sum",
    "support": "supp",
    "system": "sys",
    "table": "tbl",
    "technical": "tech",
    "technology": "tech",
    "telephone": "tel",
    "temporary": "temp",
    "territory": "terr",
    "testing": "test",
    "total": "tot",
    "transfer": "xfer",
    "translate": "trans",
    "transmission": "trans",
    "transport": "trans",
    "unique": "uniq",
    "university": "univ",
    "update": "upd",
    "upgrade": "upg",
    "user": "usr",
    "utility": "util",
    "utilities": "utils",
    "validation": "valid",
    "value": "val",
    "version": "ver",
    "video": "vid",
    "virtual": "virt",
    "volume": "vol",
    "website": "site",
    "window": "win",
    "working": "work",
    
    # Additional technical terms (massive expansion)
    "algorithm": "algo",
    "algorithms": "algos",
    "binary": "bin",
    "boolean": "bool",
    "buffer": "buf",
    "cache": "cache",
    "callback": "cb",
    "certificate": "cert",
    "character": "char",
    "characters": "chars",
    "checksum": "csum",
    "client": "cli",
    "column": "col",
    "columns": "cols",
    "command": "cmd",
    "commands": "cmds",
    "comment": "cmt",
    "comments": "cmts",
    "compiler": "comp",
    "component": "comp",
    "components": "comps",
    "compress": "zip",
    "compression": "zip",
    "compute": "calc",
    "computer": "comp",
    "concatenate": "cat",
    "condition": "cond",
    "connection": "conn",
    "constant": "const",
    "constructor": "ctor",
    "container": "ctr",
    "context": "ctx",
    "control": "ctrl",
    "controller": "ctrl",
    "convert": "conv",
    "cookie": "cookie",
    "cookies": "cookies",
    "create": "mk",
    "creation": "create",
    "criteria": "crit",
    "decimal": "dec",
    "declaration": "decl",
    "decode": "dec",
    "decrement": "dec",
    "delete": "del",
    "delimiter": "delim",
    "dependency": "dep",
    "dependencies": "deps",
    "destination": "dest",
    "destructor": "dtor",
    "device": "dev",
    "devices": "devs",
    "dictionary": "dict",
    "dimension": "dim",
    "dimensions": "dims",
    "disable": "off",
    "display": "disp",
    "download": "dl",
    "duplicate": "dup",
    "element": "elem",
    "elements": "elems",
    "enable": "on",
    "encode": "enc",
    "encrypt": "enc",
    "encryption": "enc",
    "endpoint": "ep",
    "endpoints": "eps",
    "engine": "eng",
    "exception": "ex",
    "exceptions": "exs",
    "execute": "exec",
    "execution": "exec",
    "expression": "expr",
    "expressions": "exprs",
    "factory": "fact",
    "field": "fld",
    "fields": "flds",
    "filter": "flt",
    "filters": "flts",
    "format": "fmt",
    "formula": "form",
    "fragment": "frag",
    "framework": "fw",
    "frameworks": "fws",
    "function": "fn",
    "functions": "fns",
    "generator": "gen",
    "global": "glob",
    "handle": "hdl",
    "handler": "hdlr",
    "header": "hdr",
    "headers": "hdrs",
    "height": "ht",
    "hexadecimal": "hex",
    "hierarchy": "hier",
    "hyperlink": "link",
    "identity": "id",
    "import": "imp",
    "imports": "imps",
    "increment": "inc",
    "initialize": "init",
    "instance": "inst",
    "instances": "insts",
    "instruction": "instr",
    "instructions": "instrs",
    "integer": "int",
    "integration": "integ",
    "iterator": "iter",
    "keyword": "kw",
    "keywords": "kws",
    "label": "lbl",
    "library": "lib",
    "libraries": "libs",
    "license": "lic",
    "limit": "lim",
    "limits": "lims",
    "literal": "lit",
    "load": "ld",
    "loader": "ldr",
    "localhost": "local",
    "logic": "log",
    "lookup": "lu",
    "machine": "mach",
    "manager": "mgr",
    "manifest": "mf",
    "margin": "mg",
    "margins": "mgs",
    "maximum": "max",
    "member": "mbr",
    "members": "mbrs",
    "memory": "mem",
    "merge": "merge",
    "message": "msg",
    "messages": "msgs",
    "metadata": "meta",
    "method": "mthd",
    "methods": "mthds",
    "middleware": "mw",
    "minimum": "min",
    "module": "mod",
    "modules": "mods",
    "monitor": "mon",
    "namespace": "ns",
    "namespaces": "nss",
    "navigate": "nav",
    "navigation": "nav",
    "normalize": "norm",
    "notification": "notif",
    "notifications": "notifs",
    "optimize": "opt",
    "optimization": "opt",
    "option": "opt",
    "options": "opts",
    "output": "out",
    "overflow": "ovfl",
    "override": "ovr",
    "padding": "pad",
    "parameter": "param",
    "parameters": "params",
    "parent": "par",
    "parents": "pars",
    "parse": "parse",
    "parser": "parser",
    "partition": "part",
    "password": "pwd",
    "pattern": "pat",
    "patterns": "pats",
    "payload": "pl",
    "percentage": "pct",
    "permission": "perm",
    "permissions": "perms",
    "placeholder": "ph",
    "pointer": "ptr",
    "pointers": "ptrs",
    "policy": "pol",
    "policies": "pols",
    "port": "port",
    "ports": "ports",
    "position": "pos",
    "precision": "prec",
    "predicate": "pred",
    "preference": "pref",
    "preferences": "prefs",
    "prefix": "pre",
    "prepare": "prep",
    "presentation": "pres",
    "primary": "pri",
    "privilege": "priv",
    "privileges": "privs",
    "procedure": "proc",
    "procedures": "procs",
    "process": "proc",
    "processes": "procs",
    "processor": "proc",
    "property": "prop",
    "properties": "props",
    "protocol": "proto",
    "prototype": "proto",
    "provider": "prov",
    "public": "pub",
    "publish": "pub",
    "publisher": "pub",
    "query": "qry",
    "queries": "qrys",
    "random": "rand",
    "range": "rng",
    "ranges": "rngs",
    "ratio": "ratio",
    "record": "rec",
    "records": "recs",
    "rectangle": "rect",
    "reference": "ref",
    "references": "refs",
    "register": "reg",
    "registers": "regs",
    "regular": "reg",
    "relation": "rel",
    "relations": "rels",
    "relative": "rel",
    "remove": "rm",
    "render": "rend",
    "replace": "repl",
    "repository": "repo",
    "repositories": "repos",
    "request": "req",
    "requests": "reqs",
    "require": "req",
    "required": "req",
    "resource": "res",
    "resources": "ress",
    "response": "resp",
    "responses": "resps",
    "result": "res",
    "results": "ress",
    "return": "ret",
    "revision": "rev",
    "revisions": "revs",
    "router": "rtr",
    "routing": "rtg",
    "runtime": "rt",
    "sample": "smp",
    "samples": "smps",
    "scale": "scl",
    "schedule": "sched",
    "scheduler": "sched",
    "schema": "sch",
    "schemas": "schs",
    "scope": "scp",
    "script": "scr",
    "scripts": "scrs",
    "search": "srch",
    "secure": "sec",
    "security": "sec",
    "segment": "seg",
    "segments": "segs",
    "select": "sel",
    "selector": "sel",
    "sequence": "seq",
    "sequences": "seqs",
    "serialize": "ser",
    "server": "srv",
    "servers": "srvs",
    "service": "svc",
    "services": "svcs",
    "session": "sess",
    "sessions": "sess",
    "setting": "set",
    "settings": "sets",
    "signature": "sig",
    "signatures": "sigs",
    "simulate": "sim",
    "simulation": "sim",
    "socket": "sock",
    "sockets": "socks",
    "solution": "sol",
    "solutions": "sols",
    "sort": "srt",
    "source": "src",
    "sources": "srcs",
    "specification": "spec",
    "specifications": "specs",
    "standard": "std",
    "standards": "stds",
    "start": "st",
    "statement": "stmt",
    "statements": "stmts",
    "static": "stat",
    "statistics": "stats",
    "status": "stat",
    "storage": "stor",
    "store": "str",
    "stream": "strm",
    "streams": "strms",
    "string": "str",
    "strings": "strs",
    "structure": "struct",
    "structures": "structs",
    "style": "sty",
    "styles": "stys",
    "stylesheet": "css",
    "stylesheets": "css",
    "submit": "sub",
    "subscription": "sub",
    "subscriptions": "subs",
    "success": "ok",
    "successful": "ok",
    "suffix": "suf",
    "summary": "sum",
    "support": "supp",
    "switch": "sw",
    "symbol": "sym",
    "symbols": "syms",
    "synchronize": "sync",
    "syntax": "syn",
    "system": "sys",
    "systems": "syss",
    "table": "tbl",
    "tables": "tbls",
    "target": "tgt",
    "targets": "tgts",
    "task": "task",
    "tasks": "tasks",
    "template": "tmpl",
    "templates": "tmpls",
    "temporary": "tmp",
    "test": "tst",
    "tests": "tsts",
    "text": "txt",
    "thread": "thrd",
    "threads": "thrds",
    "threshold": "thresh",
    "thresholds": "thresh",
    "timeout": "to",
    "timeouts": "tos",
    "timestamp": "ts",
    "timestamps": "tss",
    "token": "tok",
    "tokens": "toks",
    "transaction": "txn",
    "transactions": "txns",
    "transfer": "xfer",
    "transform": "xform",
    "transformation": "xform",
    "translate": "xlate",
    "translation": "xlat",
    "trigger": "trig",
    "triggers": "trigs",
    "type": "typ",
    "types": "typs",
    "unique": "uniq",
    "unknown": "unk",
    "update": "upd",
    "updates": "upds",
    "upload": "ul",
    "uploads": "uls",
    "utility": "util",
    "utilities": "utils",
    "validate": "val",
    "validation": "val",
    "validator": "val",
    "value": "val",
    "values": "vals",
    "variable": "var",
    "variables": "vars",
    "vector": "vec",
    "vectors": "vecs",
    "vendor": "vndr",
    "vendors": "vndrs",
    "version": "ver",
    "versions": "vers",
    "vertex": "vtx",
    "vertices": "vtxs",
    "viewport": "vp",
    "viewports": "vps",
    "virtual": "virt",
    "visibility": "vis",
    "visible": "vis",
    "volume": "vol",
    "volumes": "vols",
    "warning": "warn",
    "warnings": "warns",
    "weight": "wt",
    "weights": "wts",
    "width": "wd",
    "window": "win",
    "windows": "wins",
    "workflow": "wf",
    "workflows": "wfs",
    "workspace": "ws",
    "workspaces": "wss"
}

# Units and measurements
_UNITS_MEASUREMENTS = {
    "millimeter": "mm",
    "millimeters": "mm", 
    "centimeter": "cm",
    "centimeters": "cm",
    "meter": "m",
    "meters": "m",
    "kilometer": "km",
    "kilometers": "km",
    "inch": "in",
    "inches": "in",
    "foot": "ft",
    "feet": "ft",
    "yard": "yd",
    "yards": "yd",
    "mile": "mi",
    "miles": "mi",
    "ounce": "oz",
    "ounces": "oz",
    "pound": "lb",
    "pounds": "lb",
    "gram": "g",
    "grams": "g",
    "kilogram": "kg",
    "kilograms": "kg",
    "ton": "t",
    "tons": "t",
    "gallon": "gal",
    "gallons": "gal",
    "liter": "L",
    "liters": "L",
    "milliliter": "mL",
    "milliliters": "mL",
    "quart": "qt",
    "quarts": "qt",
    "pint": "pt",
    "pints": "pt",
    "cup": "c",
    "cups": "c",
    "tablespoon": "tbsp",
    "tablespoons": "tbsp",
    "teaspoon": "tsp",
    "teaspoons": "tsp",
    "second": "s",
    "seconds": "s",
    "minute": "min",
    "minutes": "min",
    "hour": "hr",
    "hours": "hr",
    "day": "d",
    "days": "d",
    "week": "wk",
    "weeks": "wk",
    "month": "mo",
    "months": "mo",
    "year": "yr",
    "years": "yr",
    "celsius": "°C",
    "fahrenheit": "°F",
    "kelvin": "K",
    "degree": "°",
    "degrees": "°",
    "percent": "%",
    "percentage": "%",
    "square meter": "m²",
    "square meters": "m²",
    "square foot": "ft²",
    "square feet": "ft²",
    "cubic meter": "m³",
    "cubic meters": "m³",
    "cubic foot": "ft³",
    "cubic feet": "ft³",
    
    # Additional scientific and technical units (massive expansion)
    "nanometer": "nm",
    "nanometers": "nm",
    "micrometer": "μm",
    "micrometers": "μm",
    "angstrom": "Å",
    "angstroms": "Å",
    "light year": "ly",
    "light years": "ly",
    "astronomical unit": "AU",
    "astronomical units": "AU",
    "parsec": "pc",
    "parsecs": "pc",
    "nautical mile": "nmi",
    "nautical miles": "nmi",
    "fathom": "fth",
    "fathoms": "fth",
    "league": "lea",
    "leagues": "lea",
    "furlong": "fur",
    "furlongs": "fur",
    "rod": "rd",
    "rods": "rd",
    "chain": "ch",
    "chains": "ch",
    "link": "lnk",
    "links": "lnk",
    "mil": "mil",
    "mils": "mil",
    
    # Weight and mass
    "picogram": "pg",
    "picograms": "pg",
    "nanogram": "ng",
    "nanograms": "ng",
    "microgram": "μg",
    "micrograms": "μg",
    "milligram": "mg",
    "milligrams": "mg",
    "carat": "ct",
    "carats": "ct",
    "grain": "gr",
    "grains": "gr",
    "dram": "dr",
    "drams": "dr",
    "pennyweight": "dwt",
    "pennyweights": "dwt",
    "troy ounce": "oz t",
    "troy ounces": "oz t",
    "stone": "st",
    "stones": "st",
    "hundredweight": "cwt",
    "hundredweights": "cwt",
    "long ton": "t",
    "long tons": "t",
    "short ton": "tn",
    "short tons": "tn",
    "metric ton": "t",
    "metric tons": "t",
    "tonne": "t",
    "tonnes": "t",
    
    # Volume
    "picoliter": "pL",
    "picoliters": "pL",
    "nanoliter": "nL",
    "nanoliters": "nL",
    "microliter": "μL",
    "microliters": "μL",
    "deciliter": "dL",
    "deciliters": "dL",
    "hectoliter": "hL",
    "hectoliters": "hL",
    "kiloliter": "kL",
    "kiloliters": "kL",
    "megaliter": "ML",
    "megaliters": "ML",
    "fluid ounce": "fl oz",
    "fluid ounces": "fl oz",
    "gill": "gi",
    "gills": "gi",
    "barrel": "bbl",
    "barrels": "bbl",
    "cord": "cd",
    "cords": "cd",
    "board foot": "bf",
    "board feet": "bf",
    "bushel": "bu",
    "bushels": "bu",
    "peck": "pk",
    "pecks": "pk",
    
    # Time
    "nanosecond": "ns",
    "nanoseconds": "ns",
    "microsecond": "μs",
    "microseconds": "μs",
    "millisecond": "ms",
    "milliseconds": "ms",
    "centisecond": "cs",
    "centiseconds": "cs",
    "decisecond": "ds",
    "deciseconds": "ds",
    "decasecond": "das",
    "decaseconds": "das",
    "hectosecond": "hs",
    "hectoseconds": "hs",
    "kilosecond": "ks",
    "kiloseconds": "ks",
    "megasecond": "Ms",
    "megaseconds": "Ms",
    "gigasecond": "Gs",
    "gigaseconds": "Gs",
    "fortnight": "fn",
    "fortnights": "fn",
    "decade": "dec",
    "decades": "dec",
    "century": "cent",
    "centuries": "cent",
    "millennium": "mill",
    "millennia": "mill",
    
    # Temperature
    "rankine": "°R",
    "degrees rankine": "°R",
    "réaumur": "°Ré",
    "degrees réaumur": "°Ré",
    "delisle": "°De",
    "degrees delisle": "°De",
    "newton": "°N",
    "degrees newton": "°N",
    "rømer": "°Rø",
    "degrees rømer": "°Rø",
    
    # Pressure
    "pascal": "Pa",
    "pascals": "Pa",
    "kilopascal": "kPa",
    "kilopascals": "kPa",
    "megapascal": "MPa",
    "megapascals": "MPa",
    "gigapascal": "GPa",
    "gigapascals": "GPa",
    "bar": "bar",
    "bars": "bar",
    "millibar": "mbar",
    "millibars": "mbar",
    "atmosphere": "atm",
    "atmospheres": "atm",
    "torr": "Torr",
    "pounds per square inch": "psi",
    "pound per square inch": "psi",
    "kilopounds per square inch": "ksi",
    "kilopound per square inch": "ksi",
    
    # Energy
    "joule": "J",
    "joules": "J",
    "kilojoule": "kJ",
    "kilojoules": "kJ",
    "megajoule": "MJ",
    "megajoules": "MJ",
    "gigajoule": "GJ",
    "gigajoules": "GJ",
    "calorie": "cal",
    "calories": "cal",
    "kilocalorie": "kcal",
    "kilocalories": "kcal",
    "british thermal unit": "BTU",
    "british thermal units": "BTU",
    "foot pound": "ft⋅lb",
    "foot pounds": "ft⋅lb",
    "electron volt": "eV",
    "electron volts": "eV",
    "kiloelectron volt": "keV",
    "kiloelectron volts": "keV",
    "megaelectron volt": "MeV",
    "megaelectron volts": "MeV",
    "gigaelectron volt": "GeV",
    "gigaelectron volts": "GeV",
    "teraelectron volt": "TeV",
    "teraelectron volts": "TeV",
    "watt hour": "Wh",
    "watt hours": "Wh",
    "kilowatt hour": "kWh",
    "kilowatt hours": "kWh",
    "megawatt hour": "MWh",
    "megawatt hours": "MWh",
    "gigawatt hour": "GWh",
    "gigawatt hours": "GWh",
    "terawatt hour": "TWh",
    "terawatt hours": "TWh",
    
    # Power
    "watt": "W",
    "watts": "W",
    "kilowatt": "kW",
    "kilowatts": "kW",
    "megawatt": "MW",
    "megawatts": "MW",
    "gigawatt": "GW",
    "gigawatts": "GW",
    "terawatt": "TW",
    "terawatts": "TW",
    "horsepower": "hp",
    "metric horsepower": "PS",
    
    # Frequency
    "hertz": "Hz",
    "kilohertz": "kHz",
    "megahertz": "MHz",
    "gigahertz": "GHz",
    "terahertz": "THz",
    "petahertz": "PHz",
    "exahertz": "EHz",
    "cycles per second": "Hz",
    "cycle per second": "Hz",
    "revolutions per minute": "rpm",
    "revolution per minute": "rpm",
    "radians per second": "rad/s",
    "radian per second": "rad/s",
    
    # Electrical
    "volt": "V",
    "volts": "V",
    "millivolt": "mV",
    "millivolts": "mV",
    "kilovolt": "kV",
    "kilovolts": "kV",
    "megavolt": "MV",
    "megavolts": "MV",
    "ampere": "A",
    "amperes": "A",
    "milliampere": "mA",
    "milliamperes": "mA",
    "microampere": "μA",
    "microamperes": "μA",
    "kiloampere": "kA",
    "kiloamperes": "kA",
    "ohm": "Ω",
    "ohms": "Ω",
    "kiloohm": "kΩ",
    "kiloohms": "kΩ",
    "megaohm": "MΩ",
    "megaohms": "MΩ",
    "farad": "F",
    "farads": "F",
    "microfarad": "μF",
    "microfarads": "μF",
    "nanofarad": "nF",
    "nanofarads": "nF",
    "picofarad": "pF",
    "picofarads": "pF",
    "henry": "H",
    "henries": "H",
    "millihenry": "mH",
    "millihenries": "mH",
    "microhenry": "μH",
    "microhenries": "μH",
    "weber": "Wb",
    "webers": "Wb",
    "tesla": "T",
    "teslas": "T",
    "gauss": "G",
    "coulomb": "C",
    "coulombs": "C",
    "siemens": "S",
    
    # Computing and data
    "bit": "b",
    "bits": "b",
    "byte": "B",
    "bytes": "B",
    "kilobit": "kb",
    "kilobits": "kb",
    "megabit": "Mb",
    "megabits": "Mb",
    "gigabit": "Gb",
    "gigabits": "Gb",
    "terabit": "Tb",
    "terabits": "Tb",
    "petabit": "Pb",
    "petabits": "Pb",
    "exabit": "Eb",
    "exabits": "Eb",
    "kibibyte": "KiB",
    "kibibytes": "KiB",
    "mebibyte": "MiB",
    "mebibytes": "MiB",
    "gibibyte": "GiB",
    "gibibytes": "GiB",
    "tebibyte": "TiB",
    "tebibytes": "TiB",
    "pebibyte": "PiB",
    "pebibytes": "PiB",
    "exbibyte": "EiB",
    "exbibytes": "EiB",
    "petabyte": "PB",
    "petabytes": "PB",
    "exabyte": "EB",
    "exabytes": "EB",
    "zettabyte": "ZB",
    "zettabytes": "ZB",
    "yottabyte": "YB",
    "yottabytes": "YB"
}

# Currency and financial terms
_CURRENCY_FINANCIAL = {
    "dollar": "$",
    "dollars": "$",
    "cent": "¢",
    "cents": "¢",
    "euro": "€",
    "euros": "€",
    "pound": "£",
    "pounds": "£",
    "yen": "¥",
    "thousand": "K",
    "million": "M",
    "billion": "B",
    "trillion": "T",
    "per annum": "p.a.",
    "per year": "/yr",
    "annually": "/yr",
    "monthly": "/mo",
    "per month": "/mo",
    "daily": "/day",
    "per day": "/day",
    "hourly": "/hr",
    "per hour": "/hr"
}

# Mathematical symbols and operators (enhanced)
_MATHEMATICAL_SYMBOLS = {
    "approximately": "~",
    "roughly": "~",
    "greater than": ">",
    "more than": ">",
    "bigger than": ">",
    "larger than": ">",
    "less than": "<",
    "fewer than": "<",
    "smaller than": "<",
    "equal to": "=",
    "equals": "=",
    "not equal": "≠",
    "not equal to": "≠",
    "greater than or equal": "≥",
    "greater than or equal to": "≥",
    "less than or equal": "≤",
    "less than or equal to": "≤",
    "plus": "+",
    "minus": "-",
    "times": "*",
    "multiplied by": "*",
    "divided by": "/",
    "per": "/",
    "infinity": "∞",
    "infinite": "∞",
    "sum": "Σ",
    "summation": "Σ",
    "product": "Π",
    "integral": "∫",
    "derivative": "d/dx",
    "partial derivative": "∂",
    "square root": "√",
    "pi": "π",
    "alpha": "α",
    "beta": "β",
    "gamma": "γ",
    "delta": "Δ",
    "theta": "θ",
    "lambda": "λ",
    "mu": "μ",
    "sigma": "σ",
    "phi": "φ",
    "omega": "ω"
}

# Scientific and technical symbols
_SCIENTIFIC_SYMBOLS = {
    "and": "&",  # Only in specific technical contexts
    "at": "@",
    "ampersand": "&",
    "copyright": "©",
    "registered": "®",
    "trademark": "™",
    "degree": "°",
    "micro": "μ",
    "nano": "n",
    "pico": "p",
    "kilo": "k",
    "mega": "M",
    "giga": "G",
    "tera": "T",
    "hertz": "Hz",
    "watts": "W",
    "volts": "V",
    "amperes": "A",
    "ohms": "Ω",
    "joules": "J",
    "calories": "cal",
    "bits per second": "bps",
    "bytes": "B",
    "kilobytes": "KB",
    "megabytes": "MB",
    "gigabytes": "GB",
    "terabytes": "TB"
}

# Expanded domain-specific acronyms
_DOMAIN_ACRONYMS = {
    # Programming & Software
    "application programming interface": "API",
    "representational state transfer": "REST", 
    "structured query language": "SQL",
    "cascading style sheets": "CSS",
    "hypertext markup language": "HTML",
    "javascript object notation": "JSON",
    "extensible markup language": "XML",
    "software development kit": "SDK",
    "integrated development environment": "IDE",
    "command line interface": "CLI",
    "graphical user interface": "GUI",
    "user interface": "UI",
    "user experience": "UX",
    "continuous integration": "CI",
    "continuous deployment": "CD",
    "version control system": "VCS",
    "pull request": "PR",
    "test driven development": "TDD",
    "object oriented programming": "OOP",
    "functional programming": "FP",
    "domain driven design": "DDD",
    "model view controller": "MVC",
    "single page application": "SPA",
    "progressive web app": "PWA",
    "content management system": "CMS",
    "customer relationship management": "CRM",
    "enterprise resource planning": "ERP",
    "extract transform load": "ETL",
    "application binary interface": "ABI",
    "just in time": "JIT",
    "ahead of time": "AOT",
    
    # Cloud & Infrastructure
    "software as a service": "SaaS",
    "platform as a service": "PaaS",
    "infrastructure as a service": "IaaS",
    "function as a service": "FaaS",
    "backend as a service": "BaaS",
    "amazon web services": "AWS",
    "google cloud platform": "GCP",
    "microsoft azure": "Azure",
    "virtual private cloud": "VPC",
    "content delivery network": "CDN",
    "load balancer": "LB",
    "auto scaling group": "ASG",
    "elastic compute cloud": "EC2",
    "simple storage service": "S3",
    "relational database service": "RDS",
    "kubernetes": "K8s",
    "docker": "Docker",
    
    # Business & Analytics
    "return on investment": "ROI",
    "return on assets": "ROA", 
    "return on equity": "ROE",
    "key performance indicator": "KPI",
    "key performance indicators": "KPIs",
    "business intelligence": "BI",
    "cost per acquisition": "CPA",
    "customer lifetime value": "CLV",
    "monthly recurring revenue": "MRR",
    "annual recurring revenue": "ARR",
    "compound annual growth rate": "CAGR",
    "year over year": "YoY",
    "quarter over quarter": "QoQ",
    "month over month": "MoM",
    "research and development": "R&D",
    "earnings before interest taxes depreciation amortization": "EBITDA",
    "initial public offering": "IPO",
    "mergers and acquisitions": "M&A",
    "chief executive officer": "CEO",
    "chief technology officer": "CTO",
    "chief financial officer": "CFO",
    "chief operating officer": "COO",
    "chief information officer": "CIO",
    "human resources": "HR",
    "information technology": "IT",
    "quality assurance": "QA",
    "search engine optimization": "SEO",
    "search engine marketing": "SEM",
    "social media marketing": "SMM",
    "pay per click": "PPC",
    "cost per click": "CPC",
    "click through rate": "CTR",
    "conversion rate optimization": "CRO",
    
    # AI/ML & Data Science
    "artificial intelligence": "AI",
    "machine learning": "ML",
    "deep learning": "DL", 
    "natural language processing": "NLP",
    "computer vision": "CV",
    "neural network": "NN",
    "convolutional neural network": "CNN",
    "recurrent neural network": "RNN",
    "long short term memory": "LSTM",
    "generative adversarial network": "GAN",
    "reinforcement learning": "RL",
    "gradient descent": "GD",
    "stochastic gradient descent": "SGD",
    "principal component analysis": "PCA",
    "support vector machine": "SVM",
    "random forest": "RF",
    "decision tree": "DT",
    "k nearest neighbors": "KNN",
    "naive bayes": "NB",
    "area under curve": "AUC",
    "receiver operating characteristic": "ROC",
    "mean squared error": "MSE",
    "root mean squared error": "RMSE",
    "mean absolute error": "MAE",
    
    # Finance & Economics
    "gross domestic product": "GDP",
    "consumer price index": "CPI",
    "federal reserve": "Fed",
    "central bank": "CB",
    "interest rate": "IR",
    "exchange traded fund": "ETF",
    "mutual fund": "MF",
    "private equity": "PE",
    "venture capital": "VC",
    "market capitalization": "market cap",
    "price to earnings": "P/E",
    "debt to equity": "D/E",
    
    # Healthcare & Medical
    "magnetic resonance imaging": "MRI",
    "computed tomography": "CT",
    "positron emission tomography": "PET",
    "electrocardiogram": "ECG",
    "blood pressure": "BP",
    "heart rate": "HR",
    "body mass index": "BMI",
    "intensive care unit": "ICU",
    "emergency room": "ER",
    "operating room": "OR",
    
    # Legal & Regulatory
    "terms of service": "ToS",
    "privacy policy": "PP",
    "non disclosure agreement": "NDA",
    "service level agreement": "SLA",
    "end user license agreement": "EULA",
    "general data protection regulation": "GDPR",
    "health insurance portability accountability act": "HIPAA",
    "securities and exchange commission": "SEC",
    "food and drug administration": "FDA",
    
    # Extended Technology & Computing
    "world wide web": "WWW",
    "internet protocol": "IP",
    "transmission control protocol": "TCP",
    "user datagram protocol": "UDP",
    "hypertext transfer protocol": "HTTP",
    "hypertext transfer protocol secure": "HTTPS",
    "file transfer protocol": "FTP",
    "simple mail transfer protocol": "SMTP",
    "post office protocol": "POP",
    "internet message access protocol": "IMAP",
    "domain name system": "DNS",
    "dynamic host configuration protocol": "DHCP",
    "simple network management protocol": "SNMP",
    "secure shell": "SSH",
    "secure sockets layer": "SSL",
    "transport layer security": "TLS",
    "virtual private network": "VPN",
    "local area network": "LAN",
    "wide area network": "WAN",
    "metropolitan area network": "MAN",
    "personal area network": "PAN",
    "wireless local area network": "WLAN",
    "wireless fidelity": "WiFi",
    "bluetooth low energy": "BLE",
    "near field communication": "NFC",
    "radio frequency identification": "RFID",
    "global positioning system": "GPS",
    "global system for mobile communications": "GSM",
    "code division multiple access": "CDMA",
    "long term evolution": "LTE",
    "fourth generation": "4G",
    "fifth generation": "5G",
    "internet of things": "IoT",
    "industrial internet of things": "IIoT",
    "edge computing": "EC",
    "fog computing": "FC",
    "quantum computing": "QC",
    "augmented reality": "AR",
    "virtual reality": "VR",
    "mixed reality": "MR",
    "extended reality": "XR",
    "computer aided design": "CAD",
    "computer aided manufacturing": "CAM",
    "computer aided engineering": "CAE",
    "computer integrated manufacturing": "CIM",
    "enterprise service bus": "ESB",
    "service oriented architecture": "SOA",
    "microservices architecture": "MSA",
    "event driven architecture": "EDA",
    "message queue": "MQ",
    "advanced message queuing protocol": "AMQP",
    "message queuing telemetry transport": "MQTT",
    "representational state transfer": "REST",
    "simple object access protocol": "SOAP",
    "remote procedure call": "RPC",
    "application performance monitoring": "APM",
    "network attached storage": "NAS",
    "storage area network": "SAN",
    "redundant array of independent disks": "RAID",
    "solid state drive": "SSD",
    "hard disk drive": "HDD",
    "optical character recognition": "OCR",
    "text to speech": "TTS",
    "speech to text": "STT",
    "automatic speech recognition": "ASR",
    
    # Extended AI/ML & Data Science
    "computer vision": "CV",
    "natural language understanding": "NLU",
    "natural language generation": "NLG",
    "optical character recognition": "OCR",
    "automated machine learning": "AutoML",
    "explainable artificial intelligence": "XAI",
    "federated learning": "FL",
    "transfer learning": "TL",
    "active learning": "AL",
    "online learning": "OL",
    "batch learning": "BL",
    "unsupervised learning": "UL",
    "supervised learning": "SL",
    "semi supervised learning": "SSL",
    "self supervised learning": "SSL",
    "few shot learning": "FSL",
    "zero shot learning": "ZSL",
    "one shot learning": "OSL",
    "meta learning": "ML",
    "curriculum learning": "CL",
    "continual learning": "CL",
    "lifelong learning": "LL",
    "catastrophic forgetting": "CF",
    "adversarial training": "AT",
    "adversarial examples": "AE",
    "generative pre trained transformer": "GPT",
    "bidirectional encoder representations from transformers": "BERT",
    "transformer": "T",
    "attention mechanism": "AM",
    "self attention": "SA",
    "multi head attention": "MHA",
    "positional encoding": "PE",
    "layer normalization": "LN",
    "batch normalization": "BN",
    "dropout": "DO",
    "residual connection": "RC",
    "skip connection": "SC",
    "feature extraction": "FE",
    "feature selection": "FS",
    "dimensionality reduction": "DR",
    "data preprocessing": "DP",
    "data augmentation": "DA",
    "synthetic data generation": "SDG",
    "data quality": "DQ",
    "data governance": "DG",
    "data lineage": "DL",
    "data catalog": "DC",
    "data warehouse": "DW",
    "data lake": "DL",
    "data mart": "DM",
    "online analytical processing": "OLAP",
    "online transaction processing": "OLTP",
    "business process management": "BPM",
    "business process modeling notation": "BPMN",
    "unified modeling language": "UML",
    "entity relationship diagram": "ERD",
    "data flow diagram": "DFD",
    
    # Extended Business & Management
    "supply chain management": "SCM",
    "supplier relationship management": "SRM",
    "vendor management system": "VMS",
    "procurement management": "PM",
    "inventory management": "IM",
    "warehouse management system": "WMS",
    "transportation management system": "TMS",
    "customer service management": "CSM",
    "field service management": "FSM",
    "project management office": "PMO",
    "program management": "PM",
    "portfolio management": "PM",
    "change management": "CM",
    "risk management": "RM",
    "compliance management": "CM",
    "vendor risk management": "VRM",
    "third party risk management": "TPRM",
    "governance risk compliance": "GRC",
    "business continuity planning": "BCP",
    "disaster recovery": "DR",
    "incident response": "IR",
    "crisis management": "CM",
    "business impact analysis": "BIA",
    "cost of goods sold": "COGS",
    "gross profit margin": "GPM",
    "net profit margin": "NPM",
    "operating profit margin": "OPM",
    "earnings before interest and taxes": "EBIT",
    "earnings per share": "EPS",
    "price to book ratio": "P/B",
    "price to sales ratio": "P/S",
    "return on capital employed": "ROCE",
    "weighted average cost of capital": "WACC",
    "net present value": "NPV",
    "internal rate of return": "IRR",
    "payback period": "PP",
    "discounted cash flow": "DCF",
    "free cash flow": "FCF",
    "operating cash flow": "OCF",
    "capital expenditure": "CAPEX",
    "operational expenditure": "OPEX",
    "working capital": "WC",
    "current ratio": "CR",
    "quick ratio": "QR",
    "debt to equity ratio": "D/E",
    "debt service coverage ratio": "DSCR",
    "asset turnover ratio": "ATR",
    "inventory turnover ratio": "ITR",
    "accounts receivable turnover": "ART",
    "accounts payable turnover": "APT",
    
    # Extended Marketing & Sales
    "customer acquisition cost": "CAC",
    "cost per lead": "CPL",
    "cost per mille": "CPM",
    "cost per action": "CPA",
    "cost per conversion": "CPC",
    "return on advertising spend": "ROAS",
    "lifetime value": "LTV",
    "net promoter score": "NPS",
    "customer satisfaction score": "CSAT",
    "customer effort score": "CES",
    "monthly active users": "MAU",
    "daily active users": "DAU",
    "weekly active users": "WAU",
    "average revenue per user": "ARPU",
    "average order value": "AOV",
    "purchase frequency": "PF",
    "cart abandonment rate": "CAR",
    "bounce rate": "BR",
    "page views": "PV",
    "unique visitors": "UV",
    "session duration": "SD",
    "time on site": "TOS",
    "engagement rate": "ER",
    "conversion funnel": "CF",
    "lead scoring": "LS",
    "marketing qualified lead": "MQL",
    "sales qualified lead": "SQL",
    "sales accepted lead": "SAL",
    "opportunity": "OPP",
    "win rate": "WR",
    "sales cycle": "SC",
    "pipeline velocity": "PV",
    "deal size": "DS",
    "quota attainment": "QA",
    "revenue per employee": "RPE",
    "annual contract value": "ACV",
    "total contract value": "TCV",
    "bookings": "BKG",
    "billings": "BIL",
    "recognized revenue": "RR",
    "deferred revenue": "DR",
    "churn rate": "CR",
    "retention rate": "RR",
    "expansion revenue": "ER",
    "upsell": "US",
    "cross sell": "CS",
    "downsell": "DS",
    
    # Extended Science & Research
    "research and development": "R&D",
    "standard operating procedure": "SOP",
    "good laboratory practice": "GLP",
    "good manufacturing practice": "GMP",
    "good clinical practice": "GCP",
    "current good manufacturing practice": "cGMP",
    "quality assurance": "QA",
    "quality control": "QC",
    "quality management system": "QMS",
    "international organization for standardization": "ISO",
    "american national standards institute": "ANSI",
    "institute of electrical and electronics engineers": "IEEE",
    "american society for testing and materials": "ASTM",
    "national institute of standards and technology": "NIST",
    "occupational safety and health administration": "OSHA",
    "environmental protection agency": "EPA",
    "material safety data sheet": "MSDS",
    "safety data sheet": "SDS",
    "personal protective equipment": "PPE",
    "hazard analysis critical control points": "HACCP",
    "statistical process control": "SPC",
    "design of experiments": "DOE",
    "failure mode and effects analysis": "FMEA",
    "root cause analysis": "RCA",
    "corrective and preventive action": "CAPA",
    "change control": "CC",
    "deviation": "DEV",
    "out of specification": "OOS",
    "out of trend": "OOT",
    "annual product review": "APR",
    "batch record": "BR",
    "master batch record": "MBR",
    "bill of materials": "BOM",
    "certificate of analysis": "COA",
    "specification": "SPEC",
    "test method": "TM",
    "validation": "VAL",
    "verification": "VER",
    "installation qualification": "IQ",
    "operational qualification": "OQ",
    "performance qualification": "PQ",
    "design qualification": "DQ",
    "user requirement specification": "URS",
    "functional requirement specification": "FRS",
    "design specification": "DS",
    "technical specification": "TS",
    
    # Extended Healthcare & Medical
    "electronic health record": "EHR",
    "electronic medical record": "EMR",
    "personal health record": "PHR",
    "health information exchange": "HIE",
    "clinical decision support system": "CDSS",
    "computerized physician order entry": "CPOE",
    "picture archiving and communication system": "PACS",
    "radiology information system": "RIS",
    "laboratory information system": "LIS",
    "hospital information system": "HIS",
    "pharmacy information system": "PIS",
    "medication administration record": "MAR",
    "adverse drug reaction": "ADR",
    "serious adverse event": "SAE",
    "adverse event": "AE",
    "medication error": "ME",
    "patient safety": "PS",
    "quality improvement": "QI",
    "clinical governance": "CG",
    "clinical audit": "CA",
    "evidence based medicine": "EBM",
    "randomized controlled trial": "RCT",
    "systematic review": "SR",
    "meta analysis": "MA",
    "case control study": "CCS",
    "cohort study": "CS",
    "cross sectional study": "CSS",
    "observational study": "OS",
    "clinical trial": "CT",
    "phase i trial": "P1",
    "phase ii trial": "P2",
    "phase iii trial": "P3",
    "phase iv trial": "P4",
    "investigational new drug": "IND",
    "new drug application": "NDA",
    "abbreviated new drug application": "ANDA",
    "biologics license application": "BLA",
    "investigational device exemption": "IDE",
    "premarket approval": "PMA",
    "premarket notification": "510K",
    "good clinical practice": "GCP",
    "institutional review board": "IRB",
    "ethics committee": "EC",
    "informed consent": "IC",
    "case report form": "CRF",
    "electronic case report form": "eCRF",
    "clinical data management": "CDM",
    "clinical data management system": "CDMS",
    "electronic data capture": "EDC",
    "source data verification": "SDV",
    "clinical monitoring": "CM",
    "clinical research associate": "CRA",
    "clinical research coordinator": "CRC",
    "principal investigator": "PI",
    "sub investigator": "SI",
    "study coordinator": "SC"
}

# Common multi-word phrases that tokenize inefficiently (expanded)
_TOKEN_EFFICIENT_REPLACEMENTS = {
    "as soon as possible": "ASAP",
    "as quickly as possible": "ASAP",
    "frequently asked questions": "FAQ",
    "frequently asked question": "FAQ",
    "step by step": "step-by-step",
    "end to end": "end-to-end",
    "state of the art": "SOTA",
    "proof of concept": "POC",
    "minimum viable product": "MVP",
    "cost benefit analysis": "CBA",
    "root cause analysis": "RCA",
    "pros and cons": "pros/cons",
    "advantages and disadvantages": "pros/cons",
    "strengths and weaknesses": "strengths/weaknesses",
    "before and after": "before/after",
    "trial and error": "trial/error",
    "supply and demand": "supply/demand",
    "research and development": "R&D",
    "operations and maintenance": "O&M",
    "terms and conditions": "T&C",
    "privacy and security": "privacy/security",
    "backup and recovery": "backup/recovery",
    "monitoring and logging": "monitoring/logging",
    "testing and debugging": "testing/debugging",
    "planning and execution": "planning/execution",
    "analysis and design": "analysis/design",
    "implementation and deployment": "impl/deploy",
    "configuration and setup": "config/setup",
    "installation and configuration": "install/config",
    "training and development": "training/dev",
    "sales and marketing": "sales/marketing",
    "recruitment and selection": "recruitment/selection",
    "buying and selling": "buying/selling",
    "import and export": "import/export",
    "input and output": "I/O",
    "question and answer": "Q&A",
    "request and response": "req/resp",
    "send and receive": "send/recv",
    "read and write": "R/W",
    "create read update delete": "CRUD",
    "black and white": "B&W",
    "old and new": "old/new",
    "large and small": "large/small",
    "fast and slow": "fast/slow",
    "high and low": "high/low",
    "left and right": "left/right",
    "up and down": "up/down",
    "on and off": "on/off",
    "yes and no": "yes/no",
    "true and false": "true/false",
    "all or nothing": "all/nothing",
    "more or less": "more/less",
    "sooner or later": "sooner/later",
    "here and there": "here/there",
    "now and then": "now/then",
    "day and night": "day/night",
    "week by week": "weekly",
    "month by month": "monthly",
    "year by year": "yearly",
    "one by one": "individually",
    "two by two": "in pairs",
    "side by side": "side-by-side",
    "face to face": "face-to-face",
    "hand in hand": "hand-in-hand",
    "word for word": "verbatim",
    "line by line": "line-by-line",
    "page by page": "page-by-page",
    "case by case": "case-by-case",
    "point by point": "point-by-point",
    
    # Extended efficiency optimizations (massive expansion)
    "application programming interface": "API",
    "user interface": "UI",
    "user experience": "UX",
    "chief executive officer": "CEO",
    "chief technology officer": "CTO",
    "chief financial officer": "CFO",
    "artificial intelligence": "AI",
    "machine learning": "ML",
    "natural language processing": "NLP",
    "frequently asked questions": "FAQ",
    "return on investment": "ROI",
    "key performance indicator": "KPI",
    "search engine optimization": "SEO",
    "content management system": "CMS",
    "customer relationship management": "CRM",
    "enterprise resource planning": "ERP",
    "software as a service": "SaaS",
    "platform as a service": "PaaS",
    "infrastructure as a service": "IaaS",
    "business to business": "B2B",
    "business to consumer": "B2C",
    "consumer to consumer": "C2C",
    "business to government": "B2G",
    "government to business": "G2B",
    "government to consumer": "G2C",
    "electronic commerce": "e-commerce",
    "electronic business": "e-business",
    "electronic learning": "e-learning",
    "electronic mail": "email",
    "electronic government": "e-government",
    "electronic health": "e-health",
    "mobile commerce": "m-commerce",
    "social commerce": "s-commerce",
    "voice over internet protocol": "VoIP",
    "internet service provider": "ISP",
    "application service provider": "ASP",
    "managed service provider": "MSP",
    "software development kit": "SDK",
    "application programming interface": "API",
    "integrated development environment": "IDE",
    "version control system": "VCS",
    "continuous integration": "CI",
    "continuous deployment": "CD",
    "test driven development": "TDD",
    "behavior driven development": "BDD",
    "domain driven design": "DDD",
    "model view controller": "MVC",
    "model view presenter": "MVP",
    "model view view model": "MVVM",
    "single page application": "SPA",
    "progressive web application": "PWA",
    "responsive web design": "RWD",
    "cross site scripting": "XSS",
    "cross site request forgery": "CSRF",
    "sql injection": "SQLi",
    "denial of service": "DoS",
    "distributed denial of service": "DDoS",
    "man in the middle": "MITM",
    "advanced persistent threat": "APT",
    "zero day exploit": "0-day",
    "proof of concept": "PoC",
    "minimum viable product": "MVP",
    "return on advertising spend": "ROAS",
    "customer lifetime value": "CLV",
    "customer acquisition cost": "CAC",
    "monthly recurring revenue": "MRR",
    "annual recurring revenue": "ARR",
    "net promoter score": "NPS",
    "customer satisfaction": "CSAT",
    "first call resolution": "FCR",
    "service level agreement": "SLA",
    "operational level agreement": "OLA",
    "recovery time objective": "RTO",
    "recovery point objective": "RPO",
    "mean time to repair": "MTTR",
    "mean time between failures": "MTBF",
    "mean time to failure": "MTTF",
    "quality of service": "QoS",
    "quality of experience": "QoE",
    "total cost of ownership": "TCO",
    "return on capital employed": "ROCE",
    "economic value added": "EVA",
    "balanced scorecard": "BSC",
    "objectives and key results": "OKR",
    "strengths weaknesses opportunities threats": "SWOT",
    "political economic social technological": "PEST",
    "political economic social technological legal environmental": "PESTLE",
    
    # Advanced compound optimizations
    "first come first served": "FCFS",
    "last in first out": "LIFO",
    "first in first out": "FIFO",
    "what you see is what you get": "WYSIWYG",
    "keep it simple stupid": "KISS",
    "do not repeat yourself": "DRY",
    "you ain't gonna need it": "YAGNI",
    "single responsibility principle": "SRP",
    "open closed principle": "OCP",
    "liskov substitution principle": "LSP",
    "interface segregation principle": "ISP",
    "dependency inversion principle": "DIP",
    "solid principles": "SOLID",
    "dont repeat yourself": "DRY",
    "principle of least privilege": "PoLP",
    "defense in depth": "DiD",
    "know your customer": "KYC",
    "anti money laundering": "AML",
    "general data protection regulation": "GDPR",
    "california consumer privacy act": "CCPA",
    "payment card industry": "PCI",
    "sarbanes oxley act": "SOX",
    "international financial reporting standards": "IFRS",
    "generally accepted accounting principles": "GAAP",
    
    # More efficiency patterns
    "best practices": "best-practices",
    "real time": "real-time",
    "real world": "real-world",
    "state of the art": "state-of-the-art",
    "cutting edge": "cutting-edge",
    "bleeding edge": "bleeding-edge",
    "mission critical": "mission-critical",
    "business critical": "business-critical",
    "time sensitive": "time-sensitive",
    "cost effective": "cost-effective",
    "user friendly": "user-friendly",
    "mobile friendly": "mobile-friendly",
    "search engine friendly": "SEO-friendly",
    "cloud native": "cloud-native",
    "cloud first": "cloud-first",
    "mobile first": "mobile-first",
    "api first": "API-first",
    "security first": "security-first",
    "privacy by design": "privacy-by-design",
    "secure by design": "secure-by-design",
    "fail safe": "fail-safe",
    "fail fast": "fail-fast",
    "plug and play": "plug-and-play",
    "drag and drop": "drag-and-drop",
    "point and click": "point-and-click",
    "copy and paste": "copy-paste",
    "cut and paste": "cut-paste",
    "search and replace": "search-replace",
    "find and replace": "find-replace",
    "trial and error": "trial-error",
    "cause and effect": "cause-effect",
    "supply and demand": "supply-demand",
    "risk and reward": "risk-reward",
    "cost and benefit": "cost-benefit",
    "give and take": "give-take",
    "push and pull": "push-pull",
    "checks and balances": "checks-balances",
    "terms and conditions": "terms-conditions",
    "rules and regulations": "rules-regulations",
    "policies and procedures": "policies-procedures",
    "roles and responsibilities": "roles-responsibilities",
    "rights and privileges": "rights-privileges",
    "assets and liabilities": "assets-liabilities",
    "income and expenses": "income-expenses",
    "revenue and costs": "revenue-costs",
    "profit and loss": "profit-loss",
    "gains and losses": "gains-losses",
    "wins and losses": "wins-losses",
    "hits and misses": "hits-misses",
    "ups and downs": "ups-downs",
    "peaks and valleys": "peaks-valleys",
    "highs and lows": "highs-lows",
    "ins and outs": "ins-outs",
    "dos and donts": "dos-donts",
    "rights and wrongs": "rights-wrongs",
    "goods and bads": "goods-bads",
    "positives and negatives": "positives-negatives",
    "pluses and minuses": "pluses-minuses",
    "advantages and disadvantages": "advantages-disadvantages",
    "benefits and drawbacks": "benefits-drawbacks",
    "opportunities and threats": "opportunities-threats",
    "challenges and solutions": "challenges-solutions",
    "problems and solutions": "problems-solutions",
    "questions and answers": "questions-answers",
    "requests and responses": "requests-responses",
    "inputs and outputs": "inputs-outputs",
    "causes and effects": "causes-effects",
    "actions and reactions": "actions-reactions",
    "efforts and results": "efforts-results",
    "plans and execution": "plans-execution",
    "strategy and tactics": "strategy-tactics",
    "theory and practice": "theory-practice",
    "knowledge and skills": "knowledge-skills",
    "education and training": "education-training",
    "learning and development": "learning-development",
    "growth and expansion": "growth-expansion",
    "improvement and optimization": "improvement-optimization",
    "innovation and creativity": "innovation-creativity",
    "design and development": "design-development",
    "build and deploy": "build-deploy",
    "test and validate": "test-validate",
    "monitor and maintain": "monitor-maintain",
    "support and maintenance": "support-maintenance",
    "backup and restore": "backup-restore",
    "save and load": "save-load",
    "export and import": "export-import",
    "upload and download": "upload-download",
    "send and receive": "send-receive",
    "publish and subscribe": "pub-sub",
    "producer and consumer": "producer-consumer",
    "client and server": "client-server",
    "frontend and backend": "frontend-backend",
    "hardware and software": "hardware-software",
    "people and process": "people-process",
    "process and technology": "process-technology",
    "tools and techniques": "tools-techniques",
    "methods and approaches": "methods-approaches",
    "standards and guidelines": "standards-guidelines",
    "best practices and recommendations": "best-practices-recommendations"
}


def abbreviate_and_symbolize(text: str) -> str:
    """Step 3: Apply comprehensive abbreviations, symbols, and acronyms.
    
    Args:
        text: Input text to process
        
    Returns:
        Text with comprehensive abbreviations and symbols applied
    """
    # Apply token-efficient replacements first (multi-word phrases)
    for phrase, replacement in _TOKEN_EFFICIENT_REPLACEMENTS.items():
        pattern = rf'\b{re.escape(phrase)}\b'
        text = re.sub(pattern, replacement, text, flags=re.IGNORECASE)
    
    # Apply domain acronyms
    for phrase, acronym in _DOMAIN_ACRONYMS.items():
        pattern = rf'\b{re.escape(phrase)}\b'
        text = re.sub(pattern, acronym, text, flags=re.IGNORECASE)
    
    # Apply currency and financial terms
    for term, symbol in _CURRENCY_FINANCIAL.items():
        pattern = rf'\b{re.escape(term)}\b'
        text = re.sub(pattern, symbol, text, flags=re.IGNORECASE)
    
    # Apply units and measurements
    for unit, abbrev in _UNITS_MEASUREMENTS.items():
        # Only replace when preceded by a number
        pattern = rf'(\d+)\s+{re.escape(unit)}\b'
        text = re.sub(pattern, rf'\1{abbrev}', text, flags=re.IGNORECASE)
    
    # Apply scientific symbols (context-aware)
    for term, symbol in _SCIENTIFIC_SYMBOLS.items():
        if term == "and":
            # Only replace "and" in specific technical contexts
            pattern = rf'\b{re.escape(term)}\b(?=\s+(?:authorization|authentication|development|data|testing|debugging|monitoring|security))'
            text = re.sub(pattern, symbol, text, flags=re.IGNORECASE)
        else:
            pattern = rf'\b{re.escape(term)}\b'
            text = re.sub(pattern, symbol, text, flags=re.IGNORECASE)
    
    # Apply mathematical symbols (very context-aware)
    for phrase, symbol in _MATHEMATICAL_SYMBOLS.items():
        # Only replace in clearly mathematical or technical contexts
        if phrase in ["plus", "minus", "times", "divided by", "equals", "equal to"]:
            pattern = rf'\b{re.escape(phrase)}\b(?=\s+(?:\d|a|an|the|by|\$|zero|one|two|three|four|five|six|seven|eight|nine))'
            text = re.sub(pattern, symbol, text, flags=re.IGNORECASE)
        elif phrase in ["greater than", "less than", "more than", "fewer than"]:
            pattern = rf'\b{re.escape(phrase)}\b(?=\s+(?:\d|zero|one|two|three|four|five|six|seven|eight|nine))'
            text = re.sub(pattern, symbol, text, flags=re.IGNORECASE)
    
    # Apply technical abbreviations
    for word, abbrev in _TECHNICAL_ABBREVIATIONS.items():
        pattern = rf'\b{re.escape(word)}\b'
        text = re.sub(pattern, abbrev, text, flags=re.IGNORECASE)
    
    # Clean up spacing
    text = re.sub(r'\s+', ' ', text)
    
    return text.strip() 